use rp_pico as bsp;

use bsp::hal::usb::UsbBus;

use usb_device::class_prelude::*;
use usbd_serial::SerialPort;

use numtoa::NumToA;

pub enum Commands {
    Reset,
}

pub struct Serial<'a> {
    serial: SerialPort<'a, UsbBus>,
    buf: [u8; 256],
}

impl<'a> Serial<'a> {
    pub fn new(usb_bus: &'a UsbBusAllocator<UsbBus>) -> Self {
        Self {
            serial: SerialPort::new(&usb_bus),
            buf: [0u8; 256],
        }
    }

    pub fn send(&mut self, i: impl NumToA<u16>) {
        let s = i.numtoa_str(10, &mut self.buf);

        // Send back to the host
        let mut wr_ptr = s.as_bytes();

        while !wr_ptr.is_empty() {
            match self.serial.write(wr_ptr) {
                Ok(len) => wr_ptr = &wr_ptr[len..],
                // On error, just drop unwritten data.
                // One possible error is Err(WouldBlock), meaning the USB
                // write buffer is full.
                Err(_) => break,
            };
        }
    }

    pub fn read(&mut self) -> Option<Commands> {
        match self.serial.read(&mut self.buf) {
            Err(_e) => None,
            Ok(0) => None,
            Ok(count) => {
                let c = count.min(self.buf.len());
                let msg = &self.buf[..c];

                match msg {
                    b"reset" => Some(Commands::Reset),
                    _ => None,
                }
            }
        }
    }
}

impl<'a> usb_device::class::UsbClass<rp_pico::hal::usb::UsbBus> for Serial<'a> {
    fn get_configuration_descriptors(
        &self,
        writer: &mut DescriptorWriter,
    ) -> usb_device::Result<()> {
        self.serial.get_configuration_descriptors(writer)
    }

    fn get_bos_descriptors(&self, writer: &mut BosWriter) -> usb_device::Result<()> {
        self.serial.get_bos_descriptors(writer)
    }

    fn get_string(&self, index: StringIndex, lang_id: u16) -> Option<&str> {
        self.serial.get_string(index, lang_id)
    }

    fn reset(&mut self) {
        self.serial.reset()
    }

    fn poll(&mut self) {
        self.serial.poll()
    }

    fn control_out(&mut self, xfer: ControlOut<rp_pico::hal::usb::UsbBus>) {
        self.serial.control_out(xfer)
    }

    fn control_in(&mut self, xfer: ControlIn<rp_pico::hal::usb::UsbBus>) {
        self.serial.control_in(xfer)
    }

    fn endpoint_setup(&mut self, addr: EndpointAddress) {
        self.serial.endpoint_setup(addr)
    }

    fn endpoint_out(&mut self, addr: EndpointAddress) {
        self.serial.endpoint_out(addr)
    }

    fn endpoint_in_complete(&mut self, addr: EndpointAddress) {
        self.serial.endpoint_in_complete(addr)
    }
}
